#MILP model with input (hour 13 to hour 24)

using JuMP
using Gurobi
using DataFrames
using Random
using XLSX

percentage = "40%"

h_df=DataFrame(XLSX.readtable("MILP_result_1-12h_$(percentage).xlsx","h")...)
h_12=Dict("z$Z"=>h_df[12,:][Z] for Z in 1:12)

@info("starting model input")
PP = Dict(
		"i1"=>Dict("Pmax"=>152,"ELnode"=>"n1", "eta"=>12.65, "NGnode"=>"m12","type"=>"GFPP"),
		"i2"=>Dict("Pmax"=>152,"ELnode"=>"n2", "eta"=>20.25, "NGnode"=>"m12","type"=>"GFPP"),
		"i3"=>Dict("Pmax"=>300,"ELnode"=>"n7", "C_E"=>45,"type"=>"TPP"),
		"i4"=>Dict("Pmax"=>591,"ELnode"=>"n13", "C_E"=>30.82,"type"=>"TPP"),
		"i5"=>Dict("Pmax"=>150,"ELnode"=>"n15", "eta"=>11.12, "NGnode"=>"m10","type"=>"GFPP"),
		"i6"=>Dict("Pmax"=>155,"ELnode"=>"n15", "eta"=>15.1, "NGnode"=>"m10","type"=>"GFPP"),
		"i7"=>Dict("Pmax"=>155,"ELnode"=>"n16", "eta"=>14.88, "NGnode"=>"m7","type"=>"GFPP"),
		"i8"=>Dict("Pmax"=>400,"ELnode"=>"n18", "C_E"=>20.84,"type"=>"TPP"),
		"i9"=>Dict("Pmax"=>400,"ELnode"=>"n21", "C_E"=>26.9,"type"=>"TPP"),
		"i10"=>Dict("Pmax"=>300,"ELnode"=> "n22", "eta"=>13.3, "NGnode"=>"m6","type"=>"GFPP"),
		"i11"=>Dict("Pmax"=>60,"ELnode"=> "n23", "eta"=>16.8, "NGnode"=>"m6","type"=>"GFPP"),
		"i12"=>Dict("Pmax"=>350,"ELnode"=>"n23", "C_E"=>32.22,"type"=>"TPP")
		)

#Injection ratio calculated to match (total W capacity/total EL demand = 20%,30%...)
inj_level=Dict("20%"=>0.31, "30%"=>0.47, "40%"=>0.63, "50%"=>0.78, "60%"=>0.94,"70%"=>1.1,"80%"=>1.25)

#Check wind injection level
Windfarm = Dict(
		"j1"=>Dict("Wmax"=>500*inj_level[percentage], "ELnode"=> "n5"),
		"j2"=>Dict("Wmax"=>1000*inj_level[percentage], "ELnode"=> "n7")
		)

WindFactor = Dict(
		"t1"=>Dict("j1"=>0.8,"j2"=>0.9),
		"t2"=>Dict("j1"=>0.9,"j2"=>0.85),
		"t3"=> Dict("j1"=>0.85,"j2"=>0.75),
		"t4"=> Dict("j1"=>0.6,"j2"=>0.7),
		"t5"=> Dict("j1"=>0.7,"j2"=>0.8),
		"t6"=>Dict("j1"=>0.68,"j2"=>0.58),
		"t7"=> Dict("j1"=>0.5,"j2"=>0.56),
		"t8"=> Dict("j1"=>0.3,"j2"=>0.4),
		"t9"=> Dict("j1"=>0.45,"j2"=>0.2),
		"t10"=>Dict("j1"=>0.85,"j2"=>0.75),
		"t11"=> Dict("j1"=>0.7,"j2"=>0.75),
		"t12"=>Dict("j1"=>0.15,"j2"=>0.35),
		"t13"=> Dict("j1"=>0.2,"j2"=>0.25),
		"t14"=>Dict("j1"=>0.4,"j2"=>0.3),
		"t15"=> Dict("j1"=>0.35,"j2"=>0.25),
		"t16"=>Dict("j1"=>0.65,"j2"=>0.7),
		"t17"=> Dict("j1"=>0.5,"j2"=>0.2),
		"t18"=>Dict("j1"=>0.25,"j2"=>0.1),
		"t19"=> Dict("j1"=>0.1,"j2"=>0.15),
		"t20"=> Dict("j1"=>0.2,"j2"=>0.15),
		"t21"=> Dict("j1"=>0.15,"j2"=>0.1),
		"t22"=> Dict("j1"=>0.15,"j2"=>0.1),
		"t23"=> Dict("j1"=>0.5,"j2"=>0.7),
		"t24"=> Dict("j1"=>0.65,"j2"=>0.77)
		)

GasSupply = Dict(
		"k1"=>Dict("Gmax"=>12000, "C_G"=>1.7, "NGnode"=>"m1"),
		"k2"=>Dict("Gmax"=>12000, "C_G"=>2.04, "NGnode"=>"m3"),
		"k3"=>Dict("Gmax"=>30000, "C_G"=>2.72, "NGnode"=>"m11")
		)

ELDemand = Dict(
		"d1" => Dict("share" => 0.038,"node"=> "n1"),
		"d2" => Dict("share" => 0.034,"node"=> "n2"),
		"d3" => Dict("share" => 0.063,"node"=> "n3"),
		"d4" => Dict("share" => 0.026,"node"=> "n4"),
		"d5" => Dict("share" => 0.025,"node"=> "n5"),
		"d6" => Dict("share" => 0.048,"node"=> "n6"),
		"d7" => Dict("share" => 0.044,"node"=> "n7"),
		"d8" => Dict("share" => 0.060,"node"=> "n8"),
		"d9" => Dict("share" => 0.061,"node"=> "n9"),
		"d10" => Dict("share" => 0.068,"node"=> "n10"),
		"d11" => Dict("share" => 0.093,"node"=> "n13"),
		"d12" => Dict("share" => 0.068,"node"=> "n14"),
		"d13" => Dict("share" => 0.111,"node"=> "n15"),
		"d14" => Dict("share" => 0.035,"node"=> "n16"),
		"d15" => Dict("share" => 0.117,"node"=> "n18"),
		"d16" => Dict("share" => 0.064,"node"=> "n19"),
		"d17" => Dict("share" => 0.045,"node"=> "n20")
		)

NGDemand = Dict(
		"d1" => Dict("share" => 0.25,"node"=> "m5"),
		"d2" => Dict("share" => 0.25,"node"=> "m7"),
		"d3" => Dict("share" => 0.35,"node"=> "m6"),
		"d4" => Dict("share" => 0.15,"node"=> "m12")
		)

Demand = Dict(
		"t1"=>Dict("EL"=>2108.73, "NG"=>7000),
		"t2"=>Dict("EL"=>2002.63, "NG"=>6700,"prev"=>"t1"),
		"t3"=>Dict("EL"=>1573.98, "NG"=>6400,"prev"=>"t2"),
		"t4"=>Dict("EL"=>1230.90, "NG"=>6500,"prev"=>"t3"),
		"t5"=>Dict("EL"=>1251.00, "NG"=>6600,"prev"=>"t4"),
		"t6"=>Dict("EL"=>1282.04, "NG"=>6700,"prev"=>"t5"),
		"t7"=>Dict("EL"=>1778.60, "NG"=>7750,"prev"=>"t6"),
		"t8"=>Dict("EL"=>2344.03, "NG"=>7800,"prev"=>"t7"),
		"t9"=>Dict("EL"=>2461.77, "NG"=>8550,"prev"=>"t8"),
		"t10"=>Dict("EL"=>2702.58, "NG"=>8700,"prev"=>"t9"),
		"t11"=>Dict("EL"=>2741.14, "NG"=>8700,"prev"=>"t10"),
		"t12"=>Dict("EL"=>2535.38, "NG"=>8550,"prev"=>"t11"),
		"t13"=>Dict("EL"=>2482.55, "NG"=>8550,"prev"=>"t12"),
		"t14"=>Dict("EL"=>2367.35, "NG"=>8550,"prev"=>"t13"),
		"t15"=>Dict("EL"=>2741.30, "NG"=>8400,"prev"=>"t14"),
		"t16"=>Dict("EL"=>2875.31, "NG"=>8550,"prev"=>"t15"),
		"t17"=>Dict("EL"=>2597.63, "NG"=>9000,"prev"=>"t16"),
		"t18"=>Dict("EL"=>2715.35, "NG"=>9000,"prev"=>"t17"),
		"t19"=>Dict("EL"=>2879.81, "NG"=>9000,"prev"=>"t18"),
		"t20"=>Dict("EL"=>2989.47, "NG"=>8700,"prev"=>"t19"),
		"t21"=>Dict("EL"=>3000.00, "NG"=>8250,"prev"=>"t20"),
		"t22"=>Dict("EL"=>2713.75, "NG"=>7500,"prev"=>"t21"),
		"t23"=>Dict("EL"=>2610.42, "NG"=>6600,"prev"=>"t22"),
		"t24"=>Dict("EL"=>2437.59, "NG"=>6700,"prev"=>"t23")
		)

Lines = Dict(
		"l1" => Dict("From" => "n1", "To" => "n2","B" => 1/0.0146, "capacity" => 175),
		"l2" => Dict("From" => "n1", "To" => "n3","B" => 1/0.2253, "capacity" => 175),
		"l3" => Dict("From" => "n1", "To" => "n5","B" => 1/0.0907, "capacity" => 500),
		"l4" => Dict("From" => "n2", "To" => "n4","B" => 1/0.1356, "capacity" => 175),
		"l5" => Dict("From" => "n2", "To" => "n6","B" => 1/0.205, "capacity" => 175),
		"l6" => Dict("From" => "n3", "To" => "n9","B" => 1/0.1271, "capacity" => 175),
		"l7" => Dict("From" => "n3", "To" => "n24","B" => 1/0.084, "capacity" => 400),
		"l8" => Dict("From" => "n4", "To" => "n9","B" => 1/0.111, "capacity" => 175),
		"l9" => Dict("From" => "n5", "To" => "n10","B" => 1/0.094, "capacity" => 500),
		"l10" => Dict("From" => "n6", "To" => "n10","B" => 1/0.0642, "capacity" => 175),
		"l11" => Dict("From" => "n7", "To" => "n8","B" => 1/0.0652, "capacity" => 1000),
		"l12" => Dict("From" => "n8", "To" => "n9","B" => 1/0.1762, "capacity" => 175),
		"l13" => Dict("From" => "n8", "To" => "n10","B" => 1/0.1762, "capacity" => 175),
		"l14" => Dict("From" => "n9", "To" => "n11","B" => 1/0.084, "capacity" => 400),
		"l15" => Dict("From" => "n9", "To" => "n12","B" => 1/0.084, "capacity" => 400),
		"l16" => Dict("From" => "n10", "To" => "n11","B" => 1/0.084, "capacity" => 400),
		"l17" => Dict("From" => "n10", "To" => "n12","B" => 1/0.084, "capacity" => 400),
		"l18" => Dict("From" => "n11", "To" => "n13","B" => 1/0.0488, "capacity" => 500),
		"l19" => Dict("From" => "n11", "To" => "n14","B" => 1/0.0426, "capacity" => 500),
		"l20" => Dict("From" => "n12", "To" => "n13","B" => 1/0.0488, "capacity" => 500),
		"l21" => Dict("From" => "n12", "To" => "n23","B" => 1/0.0985, "capacity" => 500),
		"l22" => Dict("From" => "n13", "To" => "n23","B" => 1/0.0884, "capacity" => 500),
		"l23" => Dict("From" => "n14", "To" => "n16","B" => 1/0.0594, "capacity" => 500),
		"l24" => Dict("From" => "n15", "To" => "n16","B" => 1/0.0172, "capacity" => 500),
		"l25" => Dict("From" => "n15", "To" => "n21","B" => 1/0.0249, "capacity" => 500),
		"l26" => Dict("From" => "n15", "To" => "n24","B" => 1/0.0529, "capacity" => 500),
		"l27" => Dict("From" => "n16", "To" => "n17","B" => 1/0.0263, "capacity" => 500),
		"l28" => Dict("From" => "n16", "To" => "n19","B" => 1/0.0234, "capacity" => 500),
		"l29" => Dict("From" => "n17", "To" => "n18","B" => 1/0.0143, "capacity" => 500),
		"l30" => Dict("From" => "n17", "To" => "n22","B" => 1/0.1069, "capacity" => 500),
		"l31" => Dict("From" => "n18", "To" => "n21","B" => 1/0.0132, "capacity" => 1000),
		"l32" => Dict("From" => "n19", "To" => "n20","B" => 1/0.0203, "capacity" => 1000),
		"l33" => Dict("From" => "n20", "To" => "n23","B" => 1/0.0112, "capacity" => 1000),
		"l34" => Dict("From" => "n21", "To" => "n22","B" => 1/0.0692, "capacity" => 500)
		)

Pipelines = Dict(
		"z1"=>Dict("From"=>"m1","To"=>"m2","K_f"=>28,"type"=>"passive","Gamma"=>1,"K_h"=>121,"H_ini"=>39300),
		"z2"=>Dict("From"=>"m2","To"=>"m4","K_f"=>28,"type"=>"passive","Gamma"=>1.2,"K_h"=>121,"H_ini"=>39300),
		"z3"=>Dict("From"=>"m3","To"=>"m5","K_f"=>28,"type"=>"passive","Gamma"=>1,"K_h"=>150,"H_ini"=>49300),
		"z4"=>Dict("From"=>"m4","To"=>"m5","K_f"=>21,"type"=>"passive","Gamma"=>1,"K_h"=>186,"H_ini"=>59300),
		"z5"=>Dict("From"=>"m5","To"=>"m6","K_f"=>21,"type"=>"passive","Gamma"=>1,"K_h"=>189,"H_ini"=>54300),
		"z6"=>Dict("From"=>"m4","To"=>"m7","K_f"=>21,"type"=>"passive","Gamma"=>1,"K_h"=>184,"H_ini"=>54300),
		"z7"=>Dict("From"=>"m6","To"=>"m8","K_f"=>28,"type"=>"passive","Gamma"=>1,"K_h"=>150,"H_ini"=>39300),
		"z8"=>Dict("From"=>"m7","To"=>"m8","K_f"=>21,"type"=>"passive","Gamma"=>1,"K_h"=>179,"H_ini"=>44300),
		"z9"=>Dict("From"=>"m8","To"=>"m9","K_f"=>28,"type"=>"passive","Gamma"=>1.3,"K_h"=>148,"H_ini"=>39300),
		"z10"=>Dict("From"=>"m9","To"=>"m10","K_f"=>28,"type"=>"passive","Gamma"=>1,"K_h"=>148,"H_ini"=>39300),
		"z11"=>Dict("From"=>"m10","To"=>"m11","K_f"=>28,"type"=>"passive","Gamma"=>1,"K_h"=>150,"H_ini"=>39300),
		"z12"=>Dict("From"=>"m11","To"=>"m12","K_f"=>28,"type"=>"passive","Gamma"=>1,"K_h"=>130,"H_ini"=>29300)
		)

#If Gamma=1 there is no compressor, if gamma !=1 there is a compressor forcing flow direction
#Total_H_ini = sum(Pipelines[z]["H_ini"] for z in Z)

NGPr = Dict(
		"m1"=>Dict("minPr"=> 350, "maxPr"=> 470),
		"m2"=>Dict("minPr"=> 310, "maxPr"=> 430),
		"m3"=>Dict("minPr"=> 350, "maxPr"=> 470),
		"m4"=>Dict("minPr"=> 310, "maxPr"=> 460),
		"m5"=>Dict("minPr"=> 290, "maxPr"=> 430),
		"m6"=>Dict("minPr"=> 250, "maxPr"=> 390),
		"m7"=>Dict("minPr"=> 240, "maxPr"=> 360),
		"m8"=>Dict("minPr"=> 230, "maxPr"=> 350),
		"m9"=>Dict("minPr"=> 250, "maxPr"=> 400),
		"m10"=>Dict("minPr"=> 240, "maxPr"=> 360),
		"m11"=>Dict("minPr"=> 250, "maxPr"=> 380),
		"m12"=>Dict("minPr"=> 100, "maxPr"=> 350)
		)

#Create fixed pressure point sets
Random.seed!(1000)
V_set = 50
fixPR_set = Dict("v$V"=>Dict("m$M"=>rand(NGPr["m$M"]["minPr"]:NGPr["m$M"]["maxPr"])
                        for M in 1:length(keys(NGPr))) for V in 1:V_set)
# V_choice = [15,20,33,44] #Determined from choose V plots, selected from the set of 50
V_choice = [2,4,15,20,44]
fixPR = Dict("v$V" => fixPR_set["v$V"] for V in V_choice)

#values are rounded to 1 digit!
KI_plus = Dict("z$Z"=>Dict("v$V"=>fixPR["v$V"][Pipelines["z$Z"]["From"]]^2 > fixPR["v$V"][Pipelines["z$Z"]["To"]]^2
					    ? round(Pipelines["z$Z"]["K_f"]*fixPR["v$V"][Pipelines["z$Z"]["From"]]/sqrt(fixPR["v$V"][Pipelines["z$Z"]["From"]]^2-fixPR["v$V"][Pipelines["z$Z"]["To"]]^2),digits=1)
					    : 0 for V in V_choice) #1:length(keys(fixPR))
					    for Z in 1:length(keys(NGPr)))
KO_plus = Dict("z$Z"=>Dict("v$V"=>fixPR["v$V"][Pipelines["z$Z"]["From"]]^2 > fixPR["v$V"][Pipelines["z$Z"]["To"]]^2
					    ? round(Pipelines["z$Z"]["K_f"]*fixPR["v$V"][Pipelines["z$Z"]["To"]]/sqrt(fixPR["v$V"][Pipelines["z$Z"]["From"]]^2-fixPR["v$V"][Pipelines["z$Z"]["To"]]^2),digits=1)
					    : 0 for V in V_choice)
					    for Z in 1:length(keys(NGPr)))
KI_minus = Dict("z$Z"=>Dict("v$V"=>fixPR["v$V"][Pipelines["z$Z"]["To"]]^2 > fixPR["v$V"][Pipelines["z$Z"]["From"]]^2
					    ? round(Pipelines["z$Z"]["K_f"]*fixPR["v$V"][Pipelines["z$Z"]["To"]]/sqrt(fixPR["v$V"][Pipelines["z$Z"]["To"]]^2-fixPR["v$V"][Pipelines["z$Z"]["From"]]^2),digits=1)
					    : 0 for V in V_choice)
					    for Z in 1:length(keys(NGPr)))
KO_minus = Dict("z$Z"=>Dict("v$V"=>fixPR["v$V"][Pipelines["z$Z"]["To"]]^2 > fixPR["v$V"][Pipelines["z$Z"]["From"]]^2
					    ? round(Pipelines["z$Z"]["K_f"]*fixPR["v$V"][Pipelines["z$Z"]["From"]]/sqrt(fixPR["v$V"][Pipelines["z$Z"]["To"]]^2-fixPR["v$V"][Pipelines["z$Z"]["From"]]^2),digits=1)
					    : 0 for V in V_choice)
					    for Z in 1:length(keys(NGPr)))

data = Dict(:PP=>PP, :Windfarm=>Windfarm, :WindFactor=>WindFactor, :GasSupply=>GasSupply,
			:ELDemand=> ELDemand, :NGDemand=>NGDemand, :Demand=>Demand, :Lines=>Lines, :Pipelines=>Pipelines,
			:NGPr=>NGPr, :fixPR=>fixPR, :KI_plus=>KI_plus, :KO_plus=>KO_plus, :KI_minus=>KI_minus, :KO_minus=>KO_minus,:V_choice=>V_choice)

BigM = 10000

#Define Indices
I = ["i$n" for n=1:length(keys(PP))] #Set of dispatchable power plants
C = ["i$n" for n=1:length(keys(PP)) if PP["i$n"]["type"]=="TPP"] #Subset of dispatchable thermal power plants
G = ["i$n" for n=1:length(keys(PP)) if PP["i$n"]["type"]=="GFPP"] #Subset of dispatchable natural gas power plants
J = ["j$n" for n=1:length(keys(Windfarm))] #Set of windpower units
T = ["t$n" for n=13:24] #Set of time periods
L = ["l$n" for n=1:length(keys(Lines))] #Set of electricity transmission lines
K = ["k$n" for n=1:length(keys(GasSupply))] #Set of natural gas supply units
Z = ["z$n" for n=1:length(keys(Pipelines))] #Set of natural gas pipelines
MZ = ["m$n" for n=1:length(keys(NGPr))] #Set of natural gas network nodes
ELD = ["d$n" for n=1:length(keys(ELDemand))] #Set of EL demands
NGD = ["d$n" for n=1:length(keys(NGDemand))] #Set of NG demands
N = ["n$n" for n=1:24] #Set of nodes
V = ["v$n" for n=data[:V_choice]] #Set of fixed pressure points
ind = Dict(:I=>I,:C=>C,:G=>G,:J=>J,:T=>T,:L=>L,:K=>K,:Z=>Z,:MZ=>MZ,
			   :ELD=>ELD,:NGD=>NGD,:N=>N,:V=>V)

@info("finished model input")

# Model
MILP_model = Model(Gurobi.Optimizer)
set_optimizer_attribute(MILP_model, "Threads", 24)
#set_optimizer_attribute(MILP_model, "MIPGap", 0.002) #Relative gap

#Variables
@variable(MILP_model, p[i in ind[:I], t in ind[:T]] >= 0) #PP dispatch
@variable(MILP_model, w[j in ind[:J], t in ind[:T]] >= 0) #Windfarm dispatch
@variable(MILP_model, -pi <= theta[n in ind[:N], t in ind[:T]] <= pi) #Voltage angle at node n
@variable(MILP_model, f[l in ind[:L], t in ind[:T]]) #Flow in transmission
@variable(MILP_model, g[k in ind[:K], t in ind[:T]] >= 0) #Gas dispatch
@variable(MILP_model, pr[mz in ind[:MZ], t in ind[:T]]) #Pressure at NGnodes
@variable(MILP_model, q_in_plus[z in ind[:Z], t in ind[:T]] >= 0) #Inflow natural gas rate of pipeline
@variable(MILP_model, q_in_minus[z in ind[:Z], t in ind[:T]] >= 0) #Inflow natural gas rate of pipeline
@variable(MILP_model, q_out_plus[z in ind[:Z], t in ind[:T]] >= 0) #Outflow natural gas rate of pipeline
@variable(MILP_model, q_out_minus[z in ind[:Z], t in ind[:T]] >= 0) #Outflow natural gas rate of pipeline
@variable(MILP_model, q_plus[z in ind[:Z], t in ind[:T]] >= 0) #Natural gas flow in pipeline from m to u
@variable(MILP_model, q_minus[z in ind[:Z], t in ind[:T]] >= 0) #Natural gas flow in pipeline from m to u
@variable(MILP_model, q[z in ind[:Z], t in ind[:T]]) #Natural gas flow in pipeline
@variable(MILP_model, h[z in ind[:Z], t in ind[:T]] >= 0) #Linepack in pipeline
@variable(MILP_model, y[z in ind[:Z], t in ind[:T]], Bin) #Binary variable for defining direction of the flow of NG in pipeline
@variable(MILP_model, phi_m[z in ind[:Z], t in ind[:T]] >= 0)
@variable(MILP_model, phi_u[z in ind[:Z], t in ind[:T]] >= 0)

#Objective function
@objective(MILP_model, Min,
sum(p[i,t]*data[:PP][i]["C_E"] for i in ind[:C], t in ind[:T]) #Cost of running TPPs
+sum(g[k,t]*data[:GasSupply][k]["C_G"] for k in ind[:K], t in ind[:T])) #Cost of running NGFPPs

#Constraints
@constraint(MILP_model, p_lim[i in ind[:I], t in ind[:T]], p[i,t] <= data[:PP][i]["Pmax"]) #PP supply limit
@constraint(MILP_model, w_lim[j in ind[:J], t in ind[:T]], w[j,t] <= data[:Windfarm][j]["Wmax"]*data[:WindFactor][t][j]) #WF supply forecast limit
@constraint(MILP_model, ref_bus[t in ind[:T]], theta["n1",t] == 0) #Reference bus

#Flow constraints (flow is increased x100 so that theta can take values between -pi and pi)
@constraint(MILP_model, flow_def[l in ind[:L], t in ind[:T]], f[l,t] == 100*data[:Lines][l]["B"]*(theta[data[:Lines][l]["From"],t]-theta[data[:Lines][l]["To"],t]))
@constraint(MILP_model, flow_lim[l in ind[:L], t in ind[:T]], -data[:Lines][l]["capacity"] <= f[l,t] <= data[:Lines][l]["capacity"])

#Nodal power balance
@constraint(MILP_model, power_bal[n in ind[:N], t in ind[:T]],
sum(p[i,t] for i in ind[:I] if n==data[:PP][i]["ELnode"])
+sum(w[j,t] for j in ind[:J] if n==data[:Windfarm][j]["ELnode"])
-sum(f[l,t] for l in ind[:L] if n==data[:Lines][l]["From"])
+sum(f[l,t] for l in ind[:L] if n==data[:Lines][l]["To"])
== sum(data[:ELDemand][d]["share"]*data[:Demand][t]["EL"] for d in ind[:ELD] if data[:ELDemand][d]["node"]==n))

#Gas and pressure constraints
@constraint(MILP_model, g_lim[k in ind[:K], t in ind[:T]], g[k,t] <= data[:GasSupply][k]["Gmax"]) #Gas supply limit
@constraint(MILP_model, pr_lim[mz in ind[:MZ], t in ind[:T]], data[:NGPr][mz]["minPr"] <= pr[mz,t] <= data[:NGPr][mz]["maxPr"]) #Pressure limit

#Nodal gas balance
@constraint(MILP_model, gas_bal[mz in ind[:MZ], t in ind[:T]],
sum(g[k,t] for k in ind[:K] if data[:GasSupply][k]["NGnode"] == mz)
-sum(data[:PP][i]["eta"]*p[i,t] for i in ind[:G] if data[:PP][i]["NGnode"] == mz)
-sum(q_in_plus[z,t] for z in ind[:Z] if data[:Pipelines][z]["From"] == mz)
+sum(q_out_plus[z,t] for z in ind[:Z] if data[:Pipelines][z]["To"] == mz)
-sum(q_in_minus[z,t] for z in ind[:Z] if data[:Pipelines][z]["To"] == mz)
+sum(q_out_minus[z,t] for z in ind[:Z] if data[:Pipelines][z]["From"] == mz)
==sum(data[:NGDemand][d]["share"]*data[:Demand][t]["NG"] for d in ind[:NGD] if data[:NGDemand][d]["node"] == mz))

#Compressor factor
@constraint(MILP_model, comp_def[z in ind[:Z], t in ind[:T]; data[:Pipelines][z]["Gamma"]!=1], pr[data[:Pipelines][z]["To"],t] <= data[:Pipelines][z]["Gamma"]*pr[data[:Pipelines][z]["From"],t])

#m_node = Pipelines[z]["From"]
#u_node = Pipelines[z]["To"]

@constraint(MILP_model, flow_plus1[z in ind[:Z], v in ind[:V], t in ind[:T];data[:KI_plus][z][v]>0],
q_plus[z,t] <= data[:KI_plus][z][v]*pr[data[:Pipelines][z]["From"],t]
			   -data[:KO_plus][z][v]*pr[data[:Pipelines][z]["To"],t]
			   + BigM*(1-y[z,t]))
@constraint(MILP_model, flow_minus1[z in ind[:Z], v in ind[:V], t in ind[:T];data[:KI_minus][z][v]>0],
q_minus[z,t] <= data[:KI_minus][z][v]*pr[data[:Pipelines][z]["To"],t]
			   -data[:KO_minus][z][v]*pr[data[:Pipelines][z]["From"],t]
			   + BigM*y[z,t])
#Imposing fixed direction for pipelines with compressors
@constraint(MILP_model, comp_pipelines[z in ind[:Z], t in ind[:T];data[:Pipelines][z]["Gamma"]!=1], q[z,t] >= 0)


#Average flow
@constraint(MILP_model, q_pl_def[z in ind[:Z], t in ind[:T]], q_plus[z,t] == 0.5*(q_in_plus[z,t]+q_out_plus[z,t]))
@constraint(MILP_model, q_mi_def[z in ind[:Z], t in ind[:T]], q_minus[z,t] == 0.5*(q_in_minus[z,t]+q_out_minus[z,t]))

#Bidirectional flow
@constraint(MILP_model, q_def[z in ind[:Z], t in ind[:T]], q[z,t] == q_plus[z,t]-q_minus[z,t]) #Gas flow
@constraint(MILP_model, q_pl_y[z in ind[:Z], t in ind[:T]], q_plus[z,t] <= BigM*y[z,t])
@constraint(MILP_model, q_mi_y[z in ind[:Z], t in ind[:T]], q_minus[z,t] <= BigM*(1-y[z,t]))

#Linepack average mass
@constraint(MILP_model, h_def[z in ind[:Z], t in ind[:T]], h[z,t] == data[:Pipelines][z]["K_h"]*0.5*(pr[data[:Pipelines][z]["From"],t] + pr[data[:Pipelines][z]["To"],t]))

#Linepack mass conservation with initial and final(minimum) amount of linepack
@constraint(MILP_model, h_tstart[z in ind[:Z], t in ind[:T]; t=="t13"], h[z,t] == h_12[z] + q_in_plus[z,t] - q_out_plus[z,t] + q_in_minus[z,t] - q_out_minus[z,t])
@constraint(MILP_model, h_trest[z in ind[:Z], t in ind[:T]; t!="t13"], h[z,t] == h[z,data[:Demand][t]["prev"]] + q_in_plus[z,t] - q_out_plus[z,t] + q_in_minus[z,t] - q_out_minus[z,t])
@constraint(MILP_model, h_tend[z in ind[:Z], t in ind[:T]; t==last(ind[:T])], data[:Pipelines][z]["H_ini"] <= h[z,t])


#Binary expansion
@constraint(MILP_model, phi_def1[z in ind[:Z], t in ind[:T]], phi_m[z,t]-phi_u[z,t] >= 0)
@constraint(MILP_model, phi_def2[z in ind[:Z], t in ind[:T]], pr[data[:Pipelines][z]["To"],t] - pr[data[:Pipelines][z]["From"],t] - phi_u[z,t] + phi_m[z,t] >= 0)

@constraint(MILP_model, phi_m_def1[z in ind[:Z], t in ind[:T]], -y[z,t]*BigM <= phi_m[z,t])
@constraint(MILP_model, phi_m_def2[z in ind[:Z], t in ind[:T]], phi_m[z,t] <= y[z,t]*BigM)
@constraint(MILP_model, phi_m_def3[z in ind[:Z], t in ind[:T]], -(1-y[z,t])*BigM <= phi_m[z,t]-pr[data[:Pipelines][z]["From"],t])
@constraint(MILP_model, phi_m_def4[z in ind[:Z], t in ind[:T]], phi_m[z,t]-pr[data[:Pipelines][z]["From"],t] <= (1-y[z,t])*BigM)

@constraint(MILP_model, phi_u_def1[z in ind[:Z], t in ind[:T]], -y[z,t]*BigM <= phi_u[z,t])
@constraint(MILP_model, phi_u_def2[z in ind[:Z], t in ind[:T]], phi_u[z,t] <= y[z,t]*BigM)
@constraint(MILP_model, phi_u_def3[z in ind[:Z], t in ind[:T]], -(1-y[z,t])*BigM <= phi_u[z,t]-pr[data[:Pipelines][z]["To"],t])
@constraint(MILP_model, phi_u_def4[z in ind[:Z], t in ind[:T]], phi_u[z,t]-pr[data[:Pipelines][z]["To"],t] <= (1-y[z,t])*BigM)

@constraint(MILP_model, [z in ind[:Z], t in ind[:T]], q_plus[z,t] <= 10000*(phi_m[z,t]-phi_u[z,t]))
@constraint(MILP_model, [z in ind[:Z], t in ind[:T]], q_minus[z,t] <= 10000*(pr[data[:Pipelines][z]["To"],t] - pr[data[:Pipelines][z]["From"],t] - phi_u[z,t] + phi_m[z,t]))

#Solve
@info("start solving model")
optimize!(MILP_model)

println("Termination status: $(termination_status(MILP_model))")

sol_MILP = Dict(:obj => objective_value(MILP_model),
				:comp_time =>MOI.get(MILP_model, MOI.SolveTime()),
				:p => [value.(p[i,t]) for i in ind[:I], t in ind[:T]],
				:w => [value.(w[j,t]) for j in ind[:J], t in ind[:T]],
				:theta => [value.(theta[n,t]) for n in ind[:N], t in ind[:T]],
				:f => [value.(f[l,t]) for l in ind[:L], t in ind[:T]],
				:g => [value.(g[k,t]) for k in ind[:K], t in ind[:T]],
				:pr => [value.(pr[mz,t]) for mz in ind[:MZ], t in ind[:T]],
				:q_in_plus => [value.(q_in_plus[z,t]) for z in ind[:Z], t in ind[:T]],
				:q_in_minus => [value.(q_in_minus[z,t]) for z in ind[:Z], t in ind[:T]],
				:q_out_plus => [value.(q_out_plus[z,t]) for z in ind[:Z], t in ind[:T]],
				:q_out_minus => [value.(q_out_minus[z,t]) for z in ind[:Z], t in ind[:T]],
				:q_plus => [value.(q_plus[z,t]) for z in ind[:Z], t in ind[:T]],
				:q_minus => [value.(q_minus[z,t]) for z in ind[:Z], t in ind[:T]],
				:q => [value.(q[z,t]) for z in ind[:Z], t in ind[:T]],
				:y => [value.(y[z,t]) for z in ind[:Z], t in ind[:T]],
				:h => [value.(h[z,t]) for z in ind[:Z], t in ind[:T]],
				:phi_m => [value.(phi_m[z,t]) for z in ind[:Z], t in ind[:T]],
				:phi_u => [value.(phi_u[z,t]) for z in ind[:Z], t in ind[:T]])

result_df=DataFrame(objective_value=sol_MILP[:obj],comp_time=sol_MILP[:comp_time])
p_df = DataFrame(Tables.table(transpose(sol_MILP[:p])))
w_df = DataFrame(Tables.table(transpose(sol_MILP[:w])))
theta_df = DataFrame(Tables.table(transpose(sol_MILP[:theta])))
f_df = DataFrame(Tables.table(transpose(sol_MILP[:f])))
g_df = DataFrame(Tables.table(transpose(sol_MILP[:g])))
pr_df = DataFrame(Tables.table(transpose(sol_MILP[:pr])))
q_in_plus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_in_plus])))
q_in_minus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_in_minus])))
q_out_plus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_out_plus])))
q_out_minus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_out_minus])))
q_plus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_plus])))
q_minus_df = DataFrame(Tables.table(transpose(sol_MILP[:q_minus])))
q_df = DataFrame(Tables.table(transpose(sol_MILP[:q])))
y_df = DataFrame(Tables.table(transpose(sol_MILP[:y])))
h_df = DataFrame(Tables.table(transpose(sol_MILP[:h])))
phi_m_df = DataFrame(Tables.table(transpose(sol_MILP[:phi_m])))
phi_u_df = DataFrame(Tables.table(transpose(sol_MILP[:phi_u])))
#rename!(p_df,ind[:T])
XLSX.writetable("MILP_result_13-24h_$(percentage).xlsx",
	result=(collect(eachcol(result_df)),names(result_df)),
	p = (collect(eachcol(p_df)),names(p_df)),
	w = (collect(eachcol(w_df)),names(w_df)),
	theta = (collect(eachcol(theta_df)),names(theta_df)),
	f = (collect(eachcol(f_df)),names(f_df)),
	g = (collect(eachcol(g_df)),names(g_df)),
	pr = (collect(eachcol(pr_df)),names(pr_df)),
	q_in_plus = (collect(eachcol(q_in_plus_df)),names(q_in_plus_df)),
	q_in_minus = (collect(eachcol(q_in_minus_df)),names(q_in_minus_df)),
	q_out_plus = (collect(eachcol(q_out_plus_df)),names(q_out_plus_df)),
	q_out_minus = (collect(eachcol(q_out_minus_df)),names(q_out_minus_df)),
	q_plus = (collect(eachcol(q_plus_df)),names(q_plus_df)),
	q_minus = (collect(eachcol(q_minus_df)),names(q_minus_df)),
	q = (collect(eachcol(q_df)),names(q_df)),
	y = (collect(eachcol(y_df)),names(y_df)),
	h = (collect(eachcol(h_df)),names(h_df)),
	phi_m = (collect(eachcol(phi_m_df)),names(phi_m_df)),
	phi_u = (collect(eachcol(phi_u_df)),names(phi_u_df)))
@info("Exported excel file for MILP 13-24h")
